from fasthtml.common import Div

def AspectRatio(*c, ratio: any = {1/1}, **kwargs):
    ratio = str(ratio).replace("{", "").replace("}", "")
    base_style = "position: relative; width: 100%;"
    return Div(Div(*c, style="position: absolute; inset: 0px;", **kwargs),cls="overflow-hidden", style=base_style + f" aspect-ratio:{ratio}")


from fasthtml.common import Img, Div, Span

__all__ = ["Avatar", "AvatarImage", "AvatarFallback"]

base_cls = "relative flex h-10 w-10 shrink-0 overflow-hidden rounded-full"
img_cls = "aspect-square h-full w-full hidden"
fallback_cls = "flex h-full w-full items-center justify-center rounded-full bg-muted"


def Avatar(
    *c,
    cls="",
    standard=False,
    src: str = None,
    alt: str = "Avatar Image",
    fallback: str = None,
    **kwargs,
):
    if standard:
        return Div(*c, cls=f"{base_cls} {cls}", **kwargs)
    return Div(
        AvatarImage(src=src, alt=alt),
        AvatarFallback(fallback),
        cls=f"{base_cls} {cls}",
        **kwargs,
    )


def AvatarImage(src: str = None, cls="", alt: str = "Avatar Image", **kwargs):
    return Img(
        src=src,
        alt=alt,
        cls=f"{img_cls} {cls}",
        onerror="this.remove()",
        onload="this.classList.remove('hidden')",
        **kwargs,
    )


def AvatarFallback(*c, cls="", **kwargs):
    return Span(*c, cls=f"{fallback_cls} {cls}", **kwargs)



from fasthtml.components import Div
from fasthtml.common import ft_hx

__all__ = ["Tabs", "TabsList", "TabsTrigger", "TabsContent"]


def Tabs(*c, default_value=None, **kwargs):
    return Div(*c, data_ref="tabs", data_default_value=default_value, **kwargs)


def TabsList(*c, cls=None, **kwargs):
    new_cls = "inline-flex h-10 items-center justify-center rounded-md bg-muted p-1 text-muted-foreground"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, role="tablist", **kwargs, tabindex=0)


def TabsTrigger(*c, cls=None, value=None, **kwargs):
    new_cls = "inline-flex items-center justify-center whitespace-nowrap rounded-sm px-3 py-1.5 text-sm font-medium ring-offset-background transition-all focus-visible:outline-none focus-visible:ring-2 focus-visible:ring-ring focus-visible:ring-offset-2 disabled:pointer-events-none disabled:opacity-50 data-[state=active]:bg-background data-[state=active]:text-foreground data-[state=active]:shadow-sm"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return ft_hx(
        "button",
        type="button",
        *c,
        role="tab",
        data_tab_trigger=True,
        data_value=value,
        **kwargs,
    )


def TabsContent(*c, cls=None, value=None, **kwargs):
    new_cls = "mt-2 ring-offset-background focus-visible:outline-none focus-visible:ring-2 focus-visible:ring-ring focus-visible:ring-offset-2"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(
        *c,
        role="tabpanel",
        data_tab_content=True,
        data_value=value,
        hidden=True,
        **kwargs,
    )



from fasthtml.components import Div, Thead, Tbody, Tfoot, Tr, Th, Td, Caption, Table as OgTable

__all__ = ["Table", "TableHeader", "TableBody", "TableRow", "TableHead", "TableCell", "TableCaption", "TableFooter"]

table_base_cls = "w-full caption-bottom text-sm"
table_head_cls = "[&_tr]:border-b"
table_body_cls = "[&_tr:last-child]:border-0"
table_footer_cls = "border-t bg-muted/50 font-medium [&>tr]:last:border-b-0"
table_row_cls = (
    "border-b transition-colors hover:bg-muted/50 data-[state=selected]:bg-muted"
)
table_head_cls = "h-12 px-4 text-left align-middle font-medium text-muted-foreground [&:has([role=checkbox])]:pr-0"
table_cell_cls = "p-4 align-middle [&:has([role=checkbox])]:pr-0"

def Table(*c, cls=None, **kwargs):
    new_cls = table_base_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(OgTable(*c, **kwargs), cls="relative w-full overflow-auto")


def TableHeader(*c, cls=None, **kwargs):
    new_cls = table_head_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Thead(*c, **kwargs)


def TableBody(*c, cls=None, **kwargs):
    new_cls = table_body_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Tbody(*c, **kwargs)


def TableFooter(*c, cls=None, **kwargs):
    new_cls = table_footer_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Tfoot(*c, **kwargs)


def TableRow(*c, cls=None, **kwargs):
    new_cls = table_row_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Tr(*c, **kwargs)


def TableHead(*c, cls=None, **kwargs):
    new_cls = table_head_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Th(*c, **kwargs)


def TableCell(*c, cls=None, **kwargs):
    new_cls = table_cell_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Td(*c, **kwargs)


def TableCaption(*c, cls=None, **kwargs):
    new_cls = table_cell_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Caption(*c, **kwargs)


from fasthtml.common import Div
from .button import Button
from lucide_fasthtml import Lucide

__all__ = [
    "Carousel",
    "CarouselContent",
    "CarouselItem",
    "CarouselPrevious",
    "CarouselNext",
]


def Carousel(
    *c,
    cls=None,
    items=[],
    orientation: str = "horizontal",
    autoplay: bool = False,
    duration: str = "500",
    index: int = 0,
    standard: bool = False,
    **kwargs,
):
    new_cls = "relative w-full"

    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls

    if standard:
        return Div(
            *c,
            data_autoplay="true" if autoplay else "false",
            data_orientation=orientation,
            data_ref="carousel",
            data_index=index,
            role="region",
            aria_roledescription="carousel",
            **kwargs,
        )

    carousel_items = ()
    for i in items:
        carousel_items += (CarouselItem(i),)

    return Div(
        CarouselContent(*carousel_items, duration=duration),
        CarouselPrevious(),
        CarouselNext(),
        data_autoplay="true" if autoplay else "false",
        data_orientation=orientation,
        data_ref="carousel",
        data_index=index,
        role="region",
        aria_roledescription="carousel",
        **kwargs,
    )


def CarouselContent(*c, cls=None, duration: str = "500", **kwargs):
    new_cls = "flex transition-transform ease-in-out"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(
        Div(
            *c,
            data_ref="content",
            style=f"transform:translate3d(0px, 0px, 0px);transition-duration:{duration}ms",
            **kwargs,
        ),
        cls="overflow-hidden",
    )


def CarouselItem(*c, cls=None, **kwargs):
    new_cls = ("min-w-0 shrink-0 grow-0 basis-full",)
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, data_carousel_item=True, **kwargs)


def CarouselPrevious(icon="arrow-left", cls=None, **kwargs):
    new_cls = "absolute h-8 w-8 !rounded-full"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Button(
        Lucide(icon=icon, cls="size-4"),
        variant="outline",
        size="icon",
        data_ref="prevButton",
        type="button",
        **kwargs,
    )


def CarouselNext(icon="arrow-right", cls=None, **kwargs):
    new_cls = "absolute h-8 w-8 !rounded-full"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Button(
        Lucide(icon=icon, cls="size-4"),
        variant="outline",
        size="icon",
        type="button",
        data_ref="nextButton",
        **kwargs,
    )



from fasthtml.common import Div, Hidden

__all__ = ["Slider"]


def Slider(
    cls=None,
    min: int = 0,
    max: int = 100,
    step: int = 1,
    value: int = 0,
    name: str = None,
    **kwargs,
):
    new_cls = "relative flex w-full touch-none select-none items-center"

    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls

    return Div(
        Div(
            Div(data_ref="range", cls="absolute h-full bg-primary"),
            data_ref="track",
            cls="relative h-2 w-full grow overflow-hidden rounded-full bg-secondary",
        ),
        Div(
            data_ref="thumb",
            cls="absolute top-1/2 -translate-x-1/2 -translate-y-1/2 block h-5 w-5 rounded-full border-2 border-primary bg-background ring-offset-background transition-colors focus-visible:outline-none focus-visible:ring-2 focus-visible:ring-ring focus-visible:ring-offset-2 disabled:pointer-events-none disabled:opacity-50",
        ),
        Hidden(data_ref="hidden-input", name=name, value=value),
        data_max=max,
        data_step=step,
        data_value=value,
        role="slider",
        data_min=min,
        data_ref="slider",
        aria_valuemin=min,
        aria_valuemax=max,
        aria_valuenow=value,
        **kwargs,
    )



from fasthtml.components import Div, H1, P
from lucide_fasthtml import Lucide

__all__ = ["Alert", "AlertTitle", "AlertDescription"]

alert_cls = {
    "alert": "relative w-full rounded-lg border p-4 [&>svg~*]:pl-7 [&>svg+div]:translate-y-[-3px] [&>svg]:absolute [&>svg]:left-4 [&>svg]:top-4",
    "title": "mb-1 font-medium leading-none tracking-tight",
    "description": "text-sm [&_p]:leading-relaxed",
}
alert_variants_cls = {
    "default": "bg-background text-foreground [&>svg]:text-foreground",
    "destructive": "border-destructive/50 text-destructive dark:border-destructive [&>svg]:text-destructive",
}

def AlertTitle(title: str = None, cls=None, **kwargs):
    new_cls = alert_cls["title"]
    if cls:
        new_cls += f" {cls}"

    kwargs["cls"] = new_cls
    return H1(title, **kwargs)


def AlertDescription(description: str = None, cls=None, **kwargs):
    new_cls = alert_cls["description"]
    if cls:
        new_cls += f" {cls}"

    kwargs["cls"] = new_cls
    return P(description, **kwargs)


def Alert(
    *c,
    title: str = None,
    description: str = None,
    variant: str = "default",
    cls=None,
    standard=False,
    icon: str = "chevrons-right",
    **kwargs,
):
    variants = tuple(alert_variants_cls.keys())
    assert variant in variants, f"`variant` not in {variants}"

    new_cls = f"{alert_cls['alert']} {alert_variants_cls[variant]}"
    headers = []
    if variant == "destructive":
        icon = "circle-alert"
    if cls:
        new_cls += f" {cls}"

    kwargs["cls"] = new_cls
    if standard:
        return Div(*c, **kwargs)
    headers.append(Lucide(icon=icon, cls="size-4"))
    if title:
        headers.append(AlertTitle(title))
    if description:
        headers.append(AlertDescription(description))

    return Div(*headers, *c, **kwargs)


from fasthtml.components import Div, H1, P

__all__ = ["Card", "CardHeader", "CardTitle", "CardDescription", "CardContent", "CardFooter"]

card_cls = {
    "card": "rounded-lg border bg-card text-card-foreground shadow-sm",
    "head": "flex flex-col space-y-1.5 p-6",
    "title": "text-2xl font-semibold leading-none tracking-tight",
    "description": "text-sm text-muted-foreground",
    "content": "p-6 pt-0",
    "footer": "flex items-center p-6 pt-0",
}

def CardHeader(*c, cls=None, **kwargs):
    new_cls = card_cls["head"]
    if cls:
        new_cls += f" {cls}"

    kwargs["cls"] = new_cls
    return Div(*c, **kwargs)


def CardTitle(*c, cls=None, **kwargs):
    new_cls = card_cls["title"]
    if cls:
        new_cls += f" {cls}"

    kwargs["cls"] = new_cls
    return H1(*c, **kwargs)


def CardDescription(*c, cls=None, **kwargs):
    new_cls = card_cls["description"]
    if cls:
        new_cls += f" {cls}"

    kwargs["cls"] = new_cls
    return P(*c, **kwargs)


def CardContent(*c, cls=None, **kwargs):
    new_cls = card_cls["content"]
    if cls:
        new_cls += f" {cls}"

    kwargs["cls"] = new_cls
    return Div(*c, **kwargs)


def CardFooter(*c, cls=None, **kwargs):
    new_cls = card_cls["footer"]
    if cls:
        new_cls += f" {cls}"

    kwargs["cls"] = new_cls
    return Div(*c, **kwargs)


def Card(
    *c,
    title: str = None,
    description: str = None,
    footer=None,
    cls=None,
    standard=False,
    **kwargs,
):
    new_cls = card_cls["card"]

    if cls:
        new_cls += f" {cls}"

    kwargs["cls"] = new_cls

    if standard:
        return Div(*c, **kwargs)

    header_content = []
    if title:
        header_content.append(CardTitle(title))
    if description:
        header_content.append(CardDescription(description))

    if footer:
        footer = CardFooter(footer)

    if header_content:
        header = CardHeader(
            *header_content,
        )

    return Div(header, CardContent(*c), footer, **kwargs)



from fasthtml.components import Div, P, H1, Span, ft_hx
from lucide_fasthtml import Lucide
from .button import Button

__all__ = [
    "Dialog",
    "DialogHeader",
    "DialogFooter",
    "DialogTitle",
    "DialogClose",
    "DialogDescription",
    "DialogContent",
    "DialogTrigger",
]

dialog_overlay_cls = "group-data-[state=open]:no-bg-scroll fixed inset-0 z-50 bg-black/80 group-data-[state=open]:animate-in group-data-[state=closed]:animate-out group-data-[state=closed]:fade-out-0 group-data-[state=open]:fade-in-0"
dialog_content_cls = "fixed left-[50%] top-[50%] z-50 grid w-full max-w-lg translate-x-[-50%] translate-y-[-50%] gap-4 border bg-background p-6 shadow-lg duration-200 group-data-[state=open]:animate-in group-data-[state=closed]:animate-out group-data-[state=closed]:fade-out-0 group-data-[state=open]:fade-in-0 group-data-[state=closed]:zoom-out-95 group-data-[state=open]:zoom-in-95 group-data-[state=closed]:slide-out-to-left-1/2 group-data-[state=closed]:slide-out-to-top-[48%] group-data-[state=open]:slide-in-from-left-1/2 group-data-[state=open]:slide-in-from-top-[48%] sm:rounded-lg"
dialog_closeBtn_cls = "dialog-close-button cursor-pointer active:ring absolute right-4 top-4 rounded-sm opacity-70 ring-offset-background transition-opacity hover:opacity-100 focus:outline-none focus:ring-2 focus:ring-ring focus:ring-offset-2 disabled:pointer-events-none group-data-[state=open]:bg-accent group-data-[state=open]:text-muted-foreground"
dialog_title_cls = "text-lg font-semibold leading-none tracking-tight"
dialog_description_cls = "text-sm text-muted-foreground"
dialog_header_cls = "flex flex-col space-y-1.5 text-center sm:text-left"
dialog_footer_cls = "flex flex-col-reverse sm:flex-row sm:justify-end sm:space-x-2"


def DialogHeader(*c, cls=None, **kwargs):
    new_cls = dialog_header_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, **kwargs)


def DialogFooter(*c, cls=None, **kwargs):
    new_cls = dialog_footer_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, **kwargs)


def DialogTitle(*c, cls=None, **kwargs):
    new_cls = dialog_title_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return H1(*c, **kwargs)


def DialogClose(*c, cls=None, **kwargs):
    new_cls = "dialog-close-button"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Button(*c, **kwargs)


def DialogDescription(*c, cls=None, **kwargs):
    new_cls = dialog_description_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return P(*c, **kwargs)


def DialogContent(*c, cls=None, state: str = "closed", **kwargs):
    overlay = Div(data_ref="dialog-overlay", cls=dialog_overlay_cls)
    closeBtn = ft_hx(
        "button",
        Lucide(icon="x", cls="size-4"),
        Span("Close", cls="sr-only"),
        type="button",
        tabindex=0,
        cls=dialog_closeBtn_cls,
    )
    new_cls = dialog_content_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(
        overlay,
        Div(*c, closeBtn, **kwargs),
        data_ref="dialog-portal",
        style=f"display:{'block' if state=='open' else 'none'};",
        data_state=state,
        cls="group",
    )


def DialogTrigger(*c, **kwargs):
    return Button(
        *c,
        data_ref="dialog-trigger",
        type="button",
        **kwargs,
    )


def Dialog(
    *c,
    footer: str = None,
    title: str = None,
    description: str = None,
    trigger=None,
    standard: bool = False,
    state: str = "closed",
    cls=None,
    **kwargs,
):
    new_cls = ""
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls

    dialog_trigger = None

    if standard:
        return Div(*c, data_ref="dialog", tabindex="-1", **kwargs)

    header_content = []
    if title:
        header_content.append(DialogTitle(title))
    if description:
        header_content.append(DialogDescription(description))
    if footer:
        footer = DialogFooter(footer)
    if trigger and isinstance(trigger, str):
        dialog_trigger = DialogTrigger(trigger)
    elif trigger:
        dialog_trigger = trigger

    if header_content:
        header = DialogHeader(
            *header_content,
        )

    return Div(
        dialog_trigger,
        DialogContent(
            header,
            *c,
            footer,
            state=state,
        ),
        data_ref="dialog",
        tabindex="-1",
        **kwargs,
    )



from fasthtml.common import Div, ft_hx, Span, Hr
from fasthtml.xtend import Hidden
from lucide_fasthtml import Lucide

__all__ = [
    "Select",
    "SelectContent",
    "SelectLabel",
    "SelectItem",
    "SelectSeparator",
    "SelectTrigger",
    "SelectGroup",
    "SelectValue",
]


select_cls = {
    "trigger": "flex h-10 w-full items-center justify-between rounded-md border border-input bg-background px-3 py-2 text-sm ring-offset-background placeholder:text-muted-foreground focus:outline-none focus:ring-2 focus:ring-ring focus:ring-offset-2 disabled:cursor-not-allowed disabled:opacity-50 [&>span]:line-clamp-1",
    "scroll-btn": "flex cursor-default items-center justify-center py-1",
    "content": "hidden flex-col z-50 max-h-96 min-w-[8rem] overflow-hidden rounded-md border bg-popover text-popover-foreground shadow-md group-data-[state=open]:animate-in group-data-[state=closed]:animate-out group-data-[state=closed]:fade-out-0 group-data-[state=open]:fade-in-0 group-data-[state=closed]:zoom-out-95 group-data-[state=open]:zoom-in-95 data-[popper-placement='bottom-start']:slide-in-from-top-2 data-[popper-placement='top-start']:slide-in-from-bottom-2",
    "item": "group relative flex w-full cursor-default select-none items-center rounded-sm py-1.5 pl-8 pr-2 text-sm outline-none focus:bg-accent focus:text-accent-foreground data-[disabled]:pointer-events-none data-[disabled]:opacity-50",
    "separator": "-mx-1 my-1 h-px bg-muted",
    "label": "py-1.5 pl-8 pr-2 text-sm font-semibold",
    "base": "group relative data-[state=open]:no-bg-scroll",
    "viewport": "p-1 w-full no-scrollbar",
}

# select_content_styles = "box-sizing: border-box; display: flex; flex-direction: column; outline: none; --radix-select-content-transform-origin: var(--radix-popper-transform-origin); --radix-select-content-available-width: var(--radix-popper-available-width); --radix-select-content-available-height: var(--radix-popper-available-height); --radix-select-trigger-width: var(--radix-popper-anchor-width); --radix-select-trigger-height: var(--radix-popper-anchor-height); pointer-events: auto;"


def SelectTrigger(*c, cls=None, **kwargs):
    ico = Lucide(icon="chevron-down", cls="h-4 w-4 opacity-50 shrink-0")
    new_cls = select_cls["trigger"]
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return ft_hx("button", *c, ico, type="button", data_ref="select-trigger", **kwargs)


def SelectValue(placeholder=None, cls=None, **kwargs):
    new_cls = "pointer-events-none"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Span(placeholder, data_ref="select-value", **kwargs)


def SelectScrollUpButton(cls=None, **kwargs):
    new_cls = select_cls["scroll-btn"]
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    ico = Lucide(icon="chevron-up", cls="h-4 w-4 hidden sm:flex")
    return Div(ico, aria_hidden="true", data_ref="select-scroll-up", **kwargs)


def SelectScrollDownButton(cls=None, **kwargs):
    new_cls = select_cls["scroll-btn"]
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    ico = Lucide(icon="chevron-down", cls="h-4 w-4 hidden sm:flex")
    return Div(ico, aria_hidden="true", data_ref="select-scroll-down", **kwargs)


def SelectContent(*c, cls=None, **kwargs):
    new_cls = select_cls["content"]
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(
        SelectScrollUpButton(),
        Div(
                *c,
                style="position:relative;flex:1 1 0%;overflow: auto;",
                cls=select_cls["viewport"],
                data_ref="select-viewport",
            ),
            SelectScrollDownButton(),
            tabindex="-1",
            style="box-sizing:border-box;",
            data_ref="select-content",
            **kwargs,
        )


def SelectGroup(*c, **kwargs):
    return Div(*c, role="group", **kwargs)


def SelectLabel(*c, cls=None, **kwargs):
    new_cls = select_cls["label"]
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, **kwargs)


def SelectItem(*c, cls=None, checked="false", value=None,disabled=False, **kwargs):
    new_cls = select_cls["item"]
    span_cls = "absolute left-2 hidden h-3.5 w-3.5 items-center justify-center group-data-[checked=true]:flex"
    ico = Lucide(icon="check", cls="h-4 w-4")
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(
        Span(ico, cls=span_cls),
        Span(*c),
        data_checked=checked,
        value=value,
        role="option",
        tabindex=-1,
        data_disabled=disabled,
        data_ref="select-item",
        aria_selected=checked,
        **kwargs,
    )


def SelectSeparator(**kwargs):
    cls = f"{kwargs.pop('cls', '')} {select_cls['separator']}"
    return Hr(cls=cls, **kwargs)


def Select(
    *c,
    cls=None,
    state="closed",
    placeholder: str = None,
    label: str = None,
    items: list = None,
    name=None,
    standard=False,
    **kwargs,
):
    new_cls = select_cls["base"]
    value_holder = Hidden(value="", name=name)
    render_items = ()

    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls

    if standard:
        return Div(
            value_holder,
            *c,
            data_ref="select",
            data_state=state,
            **kwargs,
        )

    if items:
        render_items = (
            SelectItem(item, value=item.lower(), name=item) for item in items
        )

    select_trigger = SelectTrigger(SelectValue(placeholder))
    select_content = (
        SelectContent(SelectGroup(SelectLabel(label), *render_items)),
    )

    return Div(
        value_holder,
        select_trigger,
        select_content,
        data_state=state,
        data_ref="select",
        **kwargs,
    )



import os
import platform
import subprocess
import sys


def get_tailwind_binary():
    system = platform.system().lower()
    machine = platform.machine().lower()

    if system == "darwin":
        os_name = "macos"
    elif system == "linux":
        os_name = "linux"
    elif system == "windows":
        os_name = "windows"
    else:
        raise ValueError(f"Unsupported operating system: {system}")

    if "arm" in machine or "aarch64" in machine:
        arch = "arm64"
    elif machine == "x86_64" or machine == "amd64":
        arch = "x64"
    elif machine == "armv7l":
        arch = "armv7"
    else:
        raise ValueError(f"Unsupported architecture: {machine}")

    extension = ".exe" if system == "windows" else ""
    return f"tailwindcss-{os_name}-{arch}{extension}"


def setup():
    if os.path.isfile("tailwindcss"):
        print("Tailwindcss is already installed")
        return
    binary_name = get_tailwind_binary()
    url = f"https://github.com/tailwindlabs/tailwindcss/releases/latest/download/{binary_name}"

    subprocess.run(["curl", "-sLO", url], check=True)
    if platform.system().lower() != "windows":
        subprocess.run(["chmod", "+x", binary_name], check=True)
    os.rename(binary_name, "tailwindcss")

    globals_css_url = "https://raw.githubusercontent.com/curtis-allan/shadcn-fasthtml-framework/main/globals.css"
    tailwind_config_url = "https://raw.githubusercontent.com/curtis-allan/shadcn-fasthtml-framework/main/tailwind.config.js"

    if not os.path.isfile("globals.css"):
        subprocess.run(["curl", "-sLO", globals_css_url], check=True)
    if not os.path.isfile("tailwind.config.js"):
        subprocess.run(["curl", "-sLO", tailwind_config_url], check=True)
    if not os.path.isfile("output.css"):
        subprocess.run(["touch" if platform.system().lower() != "windows" else "ni", "output.css"], check=True)



def watch():
    subprocess.run(
        [
            "./tailwindcss",
            "-i",
            "./globals.css",
            "-o",
            "./output.css",
            "--watch",
        ],
        check=True,
    )


def build():
    subprocess.run(
        [
            "./tailwindcss",
            "-i",
            "./globals.css",
            "-o",
            "./output.css",
            "--minify",
        ],
        check=True,
    )


def main():
    if len(sys.argv) < 2:
        print("Usage: shad4fast <command>")
        print("Available commands: setup, watch, build")
        sys.exit(1)

    command = sys.argv[1]
    if command == "setup":
        setup()
    elif command == "watch":
        watch()
    elif command == "build":
        build()
    else:
        print(f"Unknown command: {command}")
        sys.exit(1)


if __name__ == "__main__":
    main()



from fasthtml.components import Div

__all__ = ["Badge"]

badge_cls = "inline-flex items-center rounded-full border px-2.5 py-0.5 text-xs font-semibold transition-colors focus:outline-none focus:ring-2 focus:ring-ring focus:ring-offset-2"
badge_variants_cls = {
    "default": "border-transparent bg-primary text-primary-foreground hover:bg-primary/80",
    "secondary": "border-transparent bg-secondary text-secondary-foreground hover:bg-secondary/80",
    "destructive": "border-transparent bg-destructive text-destructive-foreground hover:bg-destructive/80",
    "outline": "text-foreground",
}

def Badge(*c, variant: str = "default", cls=None, **kwargs):
    variants = tuple(badge_variants_cls.keys())
    assert variant in variants, f"`variant` not in {variants}"
    new_cls = badge_cls
    new_cls += f" {badge_variants_cls[variant]}"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, **kwargs)


from fasthtml.common import Span, Input, ft_hx, Div

__all__ = ["Switch"]

switch_base_cls = "group peer inline-flex h-6 w-11 shrink-0 cursor-pointer items-center rounded-full border-2 border-transparent transition-colors focus-visible:outline-none focus-visible:ring-2 focus-visible:ring-ring focus-visible:ring-offset-2 disabled:cursor-not-allowed disabled:opacity-50 peer-checked:bg-primary bg-input peer-checked:[&>span]:translate-x-5 [&>span]:translate-x-0"
switch_thumb_cls = "pointer-events-none block h-5 w-5 rounded-full bg-background shadow-lg ring-0 transition-transform"


def Switch(checked: bool = False, cls=None, name=None, value="1", **kwargs):
    thumb = Div(
        Span(cls=switch_thumb_cls), cls=f"{switch_base_cls} {cls if cls else ''}"
    )
    value_holder = Input(
        type="checkbox",
        style="display: none;",
        name=name,
        value=value,
        checked=checked,
        cls="peer",
    )

    return ft_hx(
        "button",
        value_holder,
        thumb,
        type="button",
        onclick="this.querySelector('input').click()",
        role="switch",
        **kwargs,
    )



from fasthtml.common import Div, P, H1, Span, ft_hx
from lucide_fasthtml import Lucide
from .button import Button

__all__ = [
    "Sheet",
    "SheetClose",
    "SheetContent",
    "SheetHeader",
    "SheetTitle",
    "SheetDescription",
    "SheetFooter",
    "SheetTrigger",
]

sheet_overlay_cls = "group-data-[state=open]:no-bg-scroll fixed inset-0 z-50 bg-black/80 group-data-[state=open]:animate-in group-data-[state=closed]:animate-out group-data-[state=closed]:fade-out-0 group-data-[state=open]:fade-in-0"
sheet_variants_cls = {
    "top": "inset-x-0 top-0 border-b group-data-[state=closed]:slide-out-to-top group-data-[state=open]:slide-in-from-top",
    "bottom": "inset-x-0 bottom-0 border-t group-data-[state=closed]:slide-out-to-bottom group-data-[state=open]:slide-in-from-bottom",
    "left": "inset-y-0 left-0 h-full w-3/4 border-r group-data-[state=closed]:slide-out-to-left group-data-[state=open]:slide-in-from-left sm:max-w-sm",
    "right": "inset-y-0 right-0 h-full w-3/4 border-l group-data-[state=closed]:slide-out-to-right group-data-[state=open]:slide-in-from-right sm:max-w-sm",
}
sheet_closeBtn_cls = "sheet-close-button absolute right-4 top-4 rounded-sm opacity-70 ring-offset-background transition-opacity hover:opacity-100 focus:outline-none focus:ring-2 focus:ring-ring focus:ring-offset-2 disabled:pointer-events-none"
sheet_content_cls = "fixed z-50 gap-4 bg-background p-6 shadow-lg transition ease-in-out group-data-[state=open]:animate-in group-data-[state=closed]:animate-out group-data-[state=closed]:duration-300 group-data-[state=open]:duration-500"
sheet_header_cls = "flex flex-col space-y-2 text-center sm:text-left"
sheet_footer_cls = "flex flex-col-reverse sm:flex-row sm:justify-end sm:space-x-2"
sheet_title_cls = "text-lg font-semibold text-foreground"
sheet_description_cls = "text-sm text-muted-foreground"


def Sheet(
    *c,
    side="right",
    title=None,
    description=None,
    footer=None,
    standard=False,
    trigger=None,
    state="closed",
    cls=None,
    **kwargs,
):
    header_content = []
    sheet_trigger = None

    new_cls = None
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls

    if standard:
        return Div(*c, data_ref="sheet", role="dialog", tabindex="-1", **kwargs)

    if title:
        header_content.append(SheetTitle(title))
    if description:
        header_content.append(SheetDescription(description))
    if header_content:
        header = SheetHeader(
            *header_content,
        )
    else:
        header = None
    if footer:
        footer = SheetFooter(footer)
    else:
        footer = None
    if trigger and isinstance(trigger, str):
        sheet_trigger = SheetTrigger(trigger)
    elif trigger:
        sheet_trigger = trigger

    return Div(
        sheet_trigger,
        SheetContent(header, *c, footer, side=side, state=state),
        data_state=state,
        role="dialog",
        data_ref="sheet",
        tabindex="-1",
        **kwargs,
    )


def SheetClose(*c, cls=None, **kwargs):
    new_cls = "sheet-close-button"
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Button(
        *c,
        **kwargs,
    )


def SheetContent(*c, cls=None, state: str = "closed", side="right", **kwargs):
    overlay = Div(data_ref="sheet-overlay", cls=sheet_overlay_cls)
    new_cls = f"{sheet_content_cls} {sheet_variants_cls[side]}"
    closeBtn = ft_hx(
        "button",
        Span("Close", cls="sr-only"),
        Lucide(icon="x", cls="size-4"),
        cls=sheet_closeBtn_cls,
        type="button",
        tabindex=0,
    )
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(
        overlay,
        Div(*c, closeBtn, data_ref="sheet-content", **kwargs),
        data_ref="sheet-portal",
        style=f"display:{'block' if state=='open' else 'none'};",
        data_state=state,
        cls="group",
    )


def SheetTitle(*c, cls=None, **kwargs):
    new_cls = sheet_title_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return H1(*c, **kwargs)


def SheetDescription(*c, cls=None, **kwargs):
    new_cls = sheet_description_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return P(*c, **kwargs)


def SheetHeader(*c, cls=None, **kwargs):
    new_cls = sheet_header_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, **kwargs)


def SheetTrigger(*c, **kwargs):
    return Button(
        *c,
        type="button",
        data_ref="sheet-trigger",
        **kwargs,
    )


def SheetFooter(*c, cls=None, **kwargs):
    new_cls = sheet_footer_cls
    if cls:
        new_cls += f" {cls}"
    kwargs["cls"] = new_cls
    return Div(*c, **kwargs)
